/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package gprobocode;

import java.io.*;

import ec.*;
import ec.gp.*;
import ec.gp.koza.*;
import ec.util.*;

/**
 * <p>Title: GPRobocodeStatistics</p>
 * <p>Description: An adapting interface between Robocode and ECJ</p>
 * <p>Universita` degli Studi di Pavia</p>
 * @author Marco Piastra
 * @version 2003-5-30
 */

public class GPRobocodeStatistics extends KozaStatistics {

    static final long serialVersionUID = -642832865490504280L;

    public static final String P_SERIALIZATION = "serialize";
    public static final String DEFAULT_FILENAME = "gpRobocode";
    public static final String DEFAULT_EXTENSION = "ser";

    protected boolean serialize;

    public void setup(final EvolutionState state, final Parameter base) {
        super.setup(state, base);

        // Should we serialize out the best of each run?
        serialize = state.parameters.getBoolean(base.push(P_SERIALIZATION),
                                                null, false);
    }

    public void postEvaluationStatistics(EvolutionState state) {

        // print out the other statistics
        super.postEvaluationStatistics(state);

        // Find the best of breed plus other relevant statistics
        GPIndividual best_i = null;

        // generation feature parameters
        double meanRaw = 0.0f;
        double avgNodes = 0.0D;

        for (int x = 0; x < state.population.subpops.length; x++) {
            best_i = (GPIndividual) state.population.subpops[x].individuals[0];

            for (int y = 1;
                    y < state.population.subpops[x].individuals.length; y++) {
                if (state.population.subpops[x].individuals[y].fitness.betterThan(best_i.fitness))
                    best_i = (GPIndividual) state.population.subpops[x].individuals[y];

            }
        }

        if (serialize) {
            try {
                String filename = DEFAULT_FILENAME + state.generation + "."
                                  + DEFAULT_EXTENSION;

                FileOutputStream out = new FileOutputStream(filename);
                ObjectOutputStream outs = new ObjectOutputStream(out);

                outs.writeObject(best_i);
                outs.flush();

                outs.close();
                out.close();
            }
            catch (IOException e) {
                state.output.fatal("Serialization failure " + e);
            }
        }
    }

    public void finalStatistics(final EvolutionState state,
                                final int result) {

        // print out the other statistics
        super.finalStatistics(state, result);

        // we have only one population
        ((GPProblem) (state.evaluator.p_problem.protoCloneSimple())).describe(best_of_run[0],
                state, 0, statisticslog, Output.V_NO_GENERAL);

        if (serialize) {
            try {
                String filename = DEFAULT_FILENAME + "Best" + "."
                                  + DEFAULT_EXTENSION;

                FileOutputStream out = new FileOutputStream(filename);
                ObjectOutputStream outs = new ObjectOutputStream(out);

                outs.writeObject(best_of_run[0]);
                outs.flush();

                outs.close();
                out.close();
            }
            catch (IOException e) {
                state.output.fatal("Serialization failure " + e);
            }
        }
    }
}

