package gprobocode;

import java.io.*;
import java.util.Vector;
import ec.gp.*;


/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class GPSerialized2DOT {

    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: GPSerialized2DOT <filename>");

            return;
        }

        Object individual;

        try {
            InputStream in = new FileInputStream(args[0]);
            ObjectInputStream ins = new ObjectInputStream(in);

            individual = ins.readObject();

            ins.close();
            in.close();
        }
        catch (Exception e) {
            e.printStackTrace();

            throw new RuntimeException("Error while reading individual " + e);
        }


        if (individual instanceof GPIndividual) {
            writeGPIndividual((GPIndividual) individual, args[0]);
        }
        else {
            throw new RuntimeException("Unknown individual class "
                                       + individual.getClass());

        }

    }

    private static void writeGPIndividual(GPIndividual individual,
                                          String filename) {

        String name = getName(filename);

        for (int i = 0; i < individual.trees.length; i++) {
            try {
                FileOutputStream out = new FileOutputStream(name + "T" + i
                                                            + ".dot");
                DOTWriter writer = new DOTWriter(out, true);

                writer.writeGPTree(individual.trees[i]);

                writer.close();
                out.close();
            }
            catch (IOException e) {
                throw new RuntimeException("Write failure " + e);
            }
        }
    }

    private static String getName(String s) {
        String name = null;
        int i = s.lastIndexOf('.');

        if (i > 0 && i < s.length() - 1) {
            name = s.substring(0, i);
        }
        else {
            name = s;
        }

        return name;
    }

}


class DOTWriter extends PrintWriter {

    public DOTWriter(Writer out) {
        super(out);
    }

    public DOTWriter(Writer out, boolean autoFlush) {
        super(out, autoFlush);
    }

    public DOTWriter(OutputStream out) {
        super(out);
    }

    public DOTWriter(OutputStream out, boolean autoFlush) {
        super(out, autoFlush);
    }

    public void writeGPTree(GPTree gpTree) {

        TreeWalker treeWalker = new TreeWalker();

        treeWalker.depthFirstWalk(gpTree.child);

        writeGraph(treeWalker.nodes, treeWalker.links);
    }

    public void writeGraph(Vector nodes, Vector links) {
        println("digraph G {");
        println("rankdir=LR;");

        // First go, nodes
        Node node = null;

        for (int i = 0; i < nodes.size(); i++) {
            node = (Node) nodes.elementAt(i);

            println("n" + node.id +
            " [label=\"" + node.label + "\"];");
        }

        // Second go, links
        Link link = null;

        for (int i = 0; i < links.size(); i++) {
            link = (Link) links.elementAt(i);

            print("n" + link.from + " -> n" + link.to);
            println(" [label=\"" + link.label + "\"];");
        }

        println("}");
    }

    protected class TreeWalker {

        Vector nodes = new Vector();
        Vector links = new Vector();

        Node depthFirstWalk(GPNode gpNode) {

            Node node = new Node();

            node.id = nodes.size();
            node.label = "" + node.id + " : " + gpNode.toString();

            nodes.add(node);

            if (gpNode.children.length > 0) {
                Node child = null;
                Link link = null;

                for (int i = 0; i < gpNode.children.length; i++) {

                    child = depthFirstWalk(gpNode.children[i]);

                    link = new Link();
                    link.from = node.id;
                    link.to = child.id;
                    link.label = String.valueOf(i);

                    links.add(link);
                }
            }

            return node;
        }

    }

    public class Node {
        public int id;
        public String label;
        public String attributes;
    }

    public class Link {
        public int from;
        public int to;
        public String label;
        public String attributes;
    }

}
